package fmon.builder

import java.io._

import javafx.application.Application
import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc, Parent, Scene}
import javafx.stage.Stage

import scalafx.Includes._
import scalafx.beans.property._
import scalafx.collections.ObservableBuffer
import scalafx.scene.control._
import scalafx.scene.image.ImageView
import scalafx.scene.input.MouseEvent
import scalafx.scene.paint.Color
import scalafx.scene.layout._
import scalafx.util.Duration

import fmon.draw._

class ObsAnimation(name_ : String) {
  val name = new StringProperty(this, "name", name_)
  val frames = new ObservableBuffer[FrameToken]()
  
  override def toString = name()
}

case class FrameToken(img: StillImg) {
}

class AnimationBuilder extends Savable {
  @FXML var animationList: jfxsc.ListView[ObsAnimation] = _
  @FXML var frameList: jfxsc.ListView[FrameToken] = _
  @FXML var viewpane: javafx.scene.image.ImageView = _
  @FXML var tiles: javafx.scene.layout.TilePane = _
  
  val animations = ObservableBuffer[ObsAnimation]()
  
  val palette = new Palette(new FileInputStream(raw"C:\Users\dalyj\Documents\Design\Images\TimmahLexusX reduced\Reduced Animations\Fireball.png"), 5, 3)
  
  @FXML
  def initialize(): Unit = {
    animationList.items = animations
    animationList.selectionModel().selectedIndex.onChange(selectionChange)
    frameList.selectionModel().selectedIndex.onChange(selectFrame)
    //for (x <- 0 until palette.numAcross; y <- 0 until palette.numDown) {
    val images = palette.images
    val children = images.map(img => { 
      val imgView = new ImageView() {
        scaleX = 0.5
        scaleY = 0.5
        image = img.croppedImage()
        
      }
      imgView.handleEvent(MouseEvent.Any) {
        e: MouseEvent => if (e.clickCount == 2 && e.eventType == MouseEvent.MouseClicked) {
          addFrame(img)
        }
      }
      imgView.delegate
    })
    
    tiles.children ++= children
    tiles.prefTileWidth = 96 // TODO : I think this has to do with how the image is scaled
    tiles.prefTileHeight = 96
    
    addAnimation()
    animationList.selectionModel().selectFirst()
  }
  
  def addAnimation(): Unit = {
    animations += new ObsAnimation("Animation")
    animationList.selectionModel().selectLast()
  }
  
  def quickBuildAnimation(): Unit = {
    animations += new ObsAnimation("Animation")
    animationList.selectionModel().selectLast()
    palette.images.foreach(addFrame(_))
  }
  
  def selectionChange(): Unit = {
    frameList.items = selected.frames
  }
  
  def playAnimation(): Unit = {
    val images = frameList.items().map(f => f.img).toIndexedSeq
    val animated = new AnimatedImage(images)
    val duration = new Duration(50 * images.size)
    val animator = new SpriteAnimation(viewpane, animated, duration)
    animator.play()
  }
  
  def addFrame(): Unit = {
    //println("New frame")
    //selected.frames += new FrameToken
  }
  
  def addFrame(img: StillImg): Unit = {
    println("Image")
    selected.frames += new FrameToken(img)
  }
  
  def removeFrame(): Unit = {
    frameList.items() --= frameList.selectionModel().getSelectedItems
  }
  
  def selectFrame(): Unit = {
    val frame = frameList.selectionModel().getSelectedItem
    frame.img.draw(viewpane, 0)
  }
  
  override def saveTo(file: File): Unit = {
    
  }
  
  override def openFrom(file: File): Unit = {
    
  }
  
  private def selected: ObsAnimation = {
    animationList.selectionModel().getSelectedItem
  }
}

object AnimationBuilder {
  class ABA extends Application {
    override def start(primaryStage: Stage): Unit = {
      val frameLoader = new FXMLLoader(getClass.getResource("App.fxml"))
      val root: Parent = frameLoader.load()
      val controller = frameLoader.getController[App]()
      
      val builderLoader = new FXMLLoader(getClass.getResource("AnimationBuilder.fxml"))
      val builder: Parent = builderLoader.load()
      val builderController = builderLoader.getController[AnimationBuilder]()
      
      controller.pane.children = builder
      controller.builder = builderController
      
      val scene: Scene = new Scene(root)
      primaryStage.setScene(scene)
      primaryStage.show()
    }
  }
  
  def main(args: Array[String]): Unit = {
    Application.launch(classOf[ABA], args: _*)
  }
}